@@ -4,15 +4,18 @@ use warnings;
 use POSIX qw(WNOHANG);
 use Sys::Prctl qw(prctl);
 
+## prctl() defaults
 use constant PR_SET_PDEATHSIG => 1;
 use constant SIGHUP           => 1;
 
+## new() defaults
 use constant DEFAULT_ERROR_TIMEOUT => 10;
-use constant DEFAULT_WORKERS       => 1;
 use constant DEFAULT_REAP_TIMEOUT  => .1;
+use constant DEFAULT_SUBNAME       => 'run';
+use constant DEFAULT_WORKERS       => 1;
 use constant DEFAULT_WORKER_TOTAL  => 1;
 
-our $VERSION = 0.08;
+our $VERSION = 0.10;
 
 =head1 NAME
 
@@ -27,8 +30,9 @@ Provides a very simple, no frills fork manager.
     my $obj = My::Handler->new();
 
     my $forker = Parallel::Tiny->new(
-        handler    => $obj,
-        workers   => 4,
+        handler      => $obj,
+        subname      => 'start', # My::Handler must implement start()
+        workers      => 4,
         worker_total => 'infinite',
     );
 
@@ -44,9 +48,16 @@ Returns a new Parallel::Tiny fork manager.
 
 takes arguments as a hash or hashref with the following arguments:
 
-  handler - an object you provide which has a run() method
+  handler - an object you provide which has a run() method (unless you define "subname")
             (required)
 
+  reap_timeout - the number of seconds to wait between runs of
+            waitpid() to reap children
+            (default .1)
+
+  subname - the name of the sub you want to invoke on child spawn
+            (default 'run')
+
   workers - the number of simoltaneous forked processes you
             want to allow at one time
             (default 1)
@@ -54,47 +65,51 @@ takes arguments as a hash or hashref with the following arguments:
   worker_total - the total number of processes that you want to run
             (default 1)
 
-  reap_timeout - the number of seconds to wait between runs of
-            waitpid() to reap children
-            (default .1)
-
 You can for instance, say that you want to run 100 proccesses,
 but only 4 at a time like this:
 
     my $forker = Parallel::Tiny->new(
-        handler => $obj,
-        workers => 4,
+        handler      => $obj,
+        workers      => 4,
         worker_total => 100,
     );
 
 If you want you can provide 'infinite' for worker_total.
 If you do this, you're responsible for stopping the fork manager!
 
+Signals:
+---
+
+If the parent is sent SIGTERM, the parent will wait to reap all children.
+
+If the parent is killed before its children finish, children are configured to receive HUP.
+
 =cut
 
 sub new {
     my $class = shift;
     my $args  = ref($_[0]) ? $_[0] : {@_};
 
-    # check arguments
-    my $handler = $args->{handler};
-    die 'no handler provided' unless $handler;
-    die 'handler does not implement a run() function' unless $handler->can('run');
-
     # set some defaults
-    $args->{workers}      ||= DEFAULT_WORKERS;
     $args->{reap_timeout} ||= DEFAULT_REAP_TIMEOUT;
+    $args->{subname}      ||= DEFAULT_SUBNAME;
+    $args->{workers}      ||= DEFAULT_WORKERS;
     $args->{worker_total} ||= DEFAULT_WORKER_TOTAL;
 
     # special configuration
     undef $args->{worker_total} if $args->{worker_total} eq 'infinite';
 
+    # check args
+    die 'no handler provided' unless $args->{handler};
+    die "handler doesn't implement $args->{subname}()" unless $args->{handler}->can($args->{subname});
+
     return bless({
             _continue     => 1,
-            _handler      => $handler,
+            _handler      => $args->{handler},
             _jobs         => {},
-            _workers      => $args->{workers},
             _reap_timeout => $args->{reap_timeout},
+            _subname      => $args->{subname},
+            _workers      => $args->{workers},
             _worker_total => $args->{worker_total},
     }, $class);
 }
@@ -112,8 +127,9 @@ sub run {
 
     # setup the fork manager
     my $handler = $self->{_handler};
+    my $subname = $self->{_subname};
 
-    while ($self->waitqueue()) {
+    while ($self->_waitqueue()) {
         # parent work
         my $pid = fork();
         if ($pid) {
@@ -126,7 +142,7 @@ sub run {
         prctl(PR_SET_PDEATHSIG, SIGHUP);
         $SIG{$_} = 'DEFAULT' for keys(%SIG);
         $0 = $0 . ' - worker';
-        $handler->run();
+        $handler->$subname();
 
         # child cleanup
         exit 0;
@@ -138,13 +154,12 @@ sub run {
     return 1;
 }
 
-=item waitqueue()
+## Private
+############
 
-Blocks until a job slot is available.
-
-=cut
-
-sub waitqueue {
+## waits for another job slot to become available
+## short circuits if we've received SIGTERM or reached worker total threshold
+sub _waitqueue {
     my $self = shift;
 
     # check for any stopping conditions
@@ -160,6 +175,8 @@ sub waitqueue {
     return 1;
 }
 
+## checks all currently running jobs and reaps any that have finished, opening
+## their slot.
 sub _reapchildren {
     my $self = shift;
     foreach my $pid (keys(%{ $self->{_jobs} })) {
